@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007 Zuza Software Foundation
 # 2013 F Wolff
@@ -22,6 +20,8 @@
 """An API to provide spell checking for use in checks or elsewhere."""
 
 import logging
+from functools import lru_cache
+
 
 logger = logging.getLogger(__name__)
 
@@ -29,19 +29,20 @@ available = False
 
 try:
     # Enchant
-    from enchant import checker, DictNotFoundError, Error as EnchantError
+    from enchant import Error as EnchantError, checker
+
     available = True
     checkers = {}
 
     def _get_checker(lang):
-        if not lang in checkers:
+        if lang not in checkers:
             try:
                 checkers[lang] = checker.SpellChecker(lang)
                 # some versions only report an error when checking something
-                checkers[lang].check(u'bla')
-            except EnchantError, e:
+                checkers[lang].check("bla")
+            except EnchantError as e:
                 # sometimes this is raised instead of DictNotFoundError
-                logger.error(str(e))
+                logger.error("Dictionary not found: %s", e)
                 checkers[lang] = None
 
         return checkers[lang]
@@ -50,17 +51,17 @@ try:
         spellchecker = _get_checker(lang)
         if not spellchecker:
             return
-        spellchecker.set_text(unicode(text))
+        spellchecker.set_text(str(text))
         for err in spellchecker:
             yield err.word, err.wordpos, err.suggest()
 
+    @lru_cache(maxsize=1024)
     def simple_check(text, lang):
         spellchecker = _get_checker(lang)
         if not spellchecker:
-            return
-        spellchecker.set_text(unicode(text))
-        for err in spellchecker:
-            yield err.word
+            return []
+        spellchecker.set_text(str(text))
+        return [err.word for err in spellchecker]
 
 
 except ImportError:
