@@ -1,7 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
-# Copyright 2008 Zuza Software Foundation
+# Copyright 2008, 2016 Zuza Software Foundation
 #
 # This file is part of translate.
 #
@@ -16,18 +14,20 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""This module represents Kannada language.
+"""This module represents the Kannada language.
 
-For more information, see U{http://en.wikipedia.org/wiki/Kannada_language}
+.. seealso:: http://en.wikipedia.org/wiki/Kannada_language
 """
 
+
 from translate.lang import common
 
 
 class kn(common.Common):
     """This class represents Kannada."""
 
-    ignoretests = ["startcaps", "simplecaps"]
+    ignoretests = {
+        "all": ["simplecaps", "startcaps"],
+    }
