@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2002-2008 Zuza Software Foundation
 #
@@ -35,40 +33,34 @@ comments.
 import os
 import re
 import warnings
+from io import BytesIO
+
+from translate.misc import quote, wStringIO
 
-from translate.misc import quote
-from translate.misc import wStringIO
 
 # File normalisation
 
-normalfilenamechars = "/#.0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
-normalizetable = ""
-for i in map(chr, range(256)):
-    if i in normalfilenamechars:
-        normalizetable += i
-    else:
-        normalizetable += "_"
+normalfilenamechars = (
+    b"/#.0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
+)
 
 
-class unormalizechar(dict):
-
+class normalizechar(dict):
     def __init__(self, normalchars):
         self.normalchars = {}
         for char in normalchars:
             self.normalchars[ord(char)] = char
 
     def __getitem__(self, key):
-        return self.normalchars.get(key, u"_")
+        return self.normalchars.get(key, "_")
 
-unormalizetable = unormalizechar(normalfilenamechars.decode("ascii"))
+
+normalizetable = normalizechar(normalfilenamechars.decode("ascii"))
 
 
 def normalizefilename(filename):
     """converts any non-alphanumeric (standard roman) characters to _"""
-    if isinstance(filename, str):
-        return filename.translate(normalizetable)
-    else:
-        return filename.translate(unormalizetable)
+    return filename.translate(normalizetable)
 
 
 def makekey(ookey, long_keys):
@@ -82,11 +74,11 @@ def makekey(ookey, long_keys):
     :return: unique ascii identifier
     """
     project, sourcefile, resourcetype, groupid, localid, platform = ookey
-    sourcefile = sourcefile.replace('\\', '/')
+    sourcefile = sourcefile.replace("\\", "/")
     if long_keys:
-        sourcebase = os.path.join(project, sourcefile)
+        sourcebase = "/".join((project, sourcefile))
     else:
-        sourceparts = sourcefile.split('/')
+        sourceparts = sourcefile.split("/")
         sourcebase = "".join(sourceparts[-1:])
     if len(groupid) == 0 or len(localid) == 0:
         fullid = groupid + localid
@@ -94,9 +86,10 @@ def makekey(ookey, long_keys):
         fullid = groupid + "." + localid
     if resourcetype:
         fullid = fullid + "." + resourcetype
-    key = "%s#%s" % (sourcebase, fullid)
+    key = f"{sourcebase}#{fullid}"
     return normalizefilename(key)
 
+
 # These are functions that deal with escaping and unescaping of the text fields
 # of the SDF file. These should only be applied to the text column.
 # The fields quickhelptext and title are assumed to carry no escaping.
@@ -125,10 +118,16 @@ def escape_text(text):
 
 def unescape_text(text):
     """Unescapes SDF text to be suitable for unit consumption."""
-    return text.replace("\\\\", "\a").replace("\\n", "\n").replace("\\t", "\t").\
-           replace("\\r", "\r").replace("\a", "\\\\")
+    return (
+        text.replace("\\\\", "\a")
+        .replace("\\n", "\n")
+        .replace("\\t", "\t")
+        .replace("\\r", "\r")
+        .replace("\a", "\\\\")
+    )
 
-helptagre = re.compile('''<[/]??[a-z_\-]+?(?:| +[a-z]+?=".*?") *[/]??>''')
+
+helptagre = re.compile(r"""<[/]??[a-z_\-]+?(?:| +[a-z]+?=".*?") *[/]??>""")
 
 
 def escape_help_text(text):
@@ -141,9 +140,20 @@ def escape_help_text(text):
     text = text.replace("\\", "\\\\")
     for tag in helptagre.findall(text):
         escapethistag = False
-        for escape_tag in ["ahelp", "link", "item", "emph", "defaultinline",
-                           "switchinline", "caseinline", "variable",
-                           "bookmark_value", "image", "embedvar", "alt"]:
+        for escape_tag in [
+            "ahelp",
+            "link",
+            "item",
+            "emph",
+            "defaultinline",
+            "switchinline",
+            "caseinline",
+            "variable",
+            "bookmark_value",
+            "image",
+            "embedvar",
+            "alt",
+        ]:
             if tag.startswith("<%s" % escape_tag) or tag == "</%s>" % escape_tag:
                 escapethistag = True
         if tag in ["<br/>", "<help-id-missing/>"]:
@@ -156,51 +166,90 @@ def escape_help_text(text):
 
 def unescape_help_text(text):
     """Unescapes normal text to be suitable for writing to the SDF file."""
-    return text.replace(r"\<", "<").replace(r"\>", ">").replace(r'\"', '"').replace(r"\\", "\\")
+    return (
+        text.replace(r"\<", "<")
+        .replace(r"\>", ">")
+        .replace(r"\"", '"')
+        .replace(r"\\", "\\")
+    )
 
 
-def encode_if_needed_utf8(text):
-    """Encode a Unicode string the the specified encoding"""
-    if isinstance(text, unicode):
-        return text.encode('UTF-8')
-    return text
-
-
-class ooline(object):
+class ooline:
     """this represents one line, one translation in an .oo file"""
 
     def __init__(self, parts=None):
         """construct an ooline from its parts"""
         if parts is None:
-            self.project, self.sourcefile, self.dummy, self.resourcetype, \
-                self.groupid, self.localid, self.helpid, self.platform, \
-                self.width, self.languageid, self.text, self.helptext, \
-                self.quickhelptext, self.title, self.timestamp = [""] * 15
+            (
+                self.project,
+                self.sourcefile,
+                self.dummy,
+                self.resourcetype,
+                self.groupid,
+                self.localid,
+                self.helpid,
+                self.platform,
+                self.width,
+                self.languageid,
+                self.text,
+                self.helptext,
+                self.quickhelptext,
+                self.title,
+                self.timestamp,
+            ) = [""] * 15
         else:
             self.setparts(parts)
 
     def setparts(self, parts):
         """create a line from its tab-delimited parts"""
         if len(parts) != 15:
-            warnings.warn("oo line contains %d parts, it should contain 15: %r" % \
-                    (len(parts), parts))
+            warnings.warn(
+                "oo line contains %d parts, it should contain 15: %r"
+                % (len(parts), parts)
+            )
             newparts = list(parts)
             if len(newparts) < 15:
                 newparts = newparts + [""] * (15 - len(newparts))
             else:
                 newparts = newparts[:15]
             parts = tuple(newparts)
-        self.project, self.sourcefile, self.dummy, self.resourcetype, \
-            self.groupid, self.localid, self.helpid, self.platform, \
-            self.width, self.languageid, self._text, self.helptext, \
-            self.quickhelptext, self.title, self.timestamp = parts
+        (
+            self.project,
+            self.sourcefile,
+            self.dummy,
+            self.resourcetype,
+            self.groupid,
+            self.localid,
+            self.helpid,
+            self.platform,
+            self.width,
+            self.languageid,
+            self._text,
+            self.helptext,
+            self.quickhelptext,
+            self.title,
+            self.timestamp,
+        ) = parts
 
     def getparts(self):
         """return a list of parts in this line"""
-        return (self.project, self.sourcefile, self.dummy, self.resourcetype,
-                self.groupid, self.localid, self.helpid, self.platform,
-                self.width, self.languageid, self._text, self.helptext,
-                self.quickhelptext, self.title, self.timestamp)
+        return (
+            self.project,
+            self.sourcefile,
+            self.dummy,
+            self.resourcetype,
+            self.groupid,
+            self.localid,
+            self.helpid,
+            self.platform,
+            self.width,
+            self.languageid,
+            self._text,
+            self.helptext,
+            self.quickhelptext,
+            self.title,
+            self.timestamp,
+        )
 
     def gettext(self):
         """Obtains the text column and handle escaping."""
@@ -215,11 +264,12 @@ class ooline(object):
             self._text = escape_help_text(text)
         else:
             self._text = escape_text(text)
+
     text = property(gettext, settext)
 
     def __str__(self):
-        """convert to a string. double check that unicode is handled"""
-        return encode_if_needed_utf8(self.getoutput())
+        """convert to a string."""
+        return self.getoutput()
 
     def getoutput(self):
         """return a line in tab-delimited form"""
@@ -228,8 +278,14 @@ class ooline(object):
 
     def getkey(self):
         """get the key that identifies the resource"""
-        return (self.project, self.sourcefile, self.resourcetype, self.groupid,
-                self.localid, self.platform)
+        return (
+            self.project,
+            self.sourcefile,
+            self.resourcetype,
+            self.groupid,
+            self.localid,
+            self.platform,
+        )
 
 
 class oounit:
@@ -246,8 +302,8 @@ class oounit:
         self.lines.append(line)
 
     def __str__(self):
-        """convert to a string. double check that unicode is handled"""
-        return encode_if_needed_utf8(self.getoutput())
+        """convert to a string."""
+        return self.getoutput()
 
     def getoutput(self, skip_source=False, fallback_lang=None):
         """return the lines in tab-delimited form"""
@@ -260,12 +316,14 @@ class oounit:
                 lines = [new_line]
         else:
             lines = self.lines
-        return "\r\n".join([str(line) for line in lines])
+        return "\r\n".join(str(line) for line in lines)
 
 
 class oofile:
     """this represents an entire .oo file"""
+
     UnitClass = oounit
+    encoding = "utf-8"
 
     def __init__(self, input=None):
         """constructs the oofile"""
@@ -293,12 +351,13 @@ class oofile:
     def parse(self, input):
         """parses lines and adds them to the file"""
         if not self.filename:
-            self.filename = getattr(input, 'name', '')
+            self.filename = getattr(input, "name", "")
         if hasattr(input, "read"):
             src = input.read()
             input.close()
         else:
             src = input
+        src = src.decode(self.encoding)
         for line in src.split("\n"):
             line = quote.rstripeol(line)
             if not line:
@@ -307,40 +366,55 @@ class oofile:
             thisline = ooline(parts)
             self.addline(thisline)
 
-    def __str__(self, skip_source=False, fallback_lang=None):
+    def __bytes__(self):
+        out = BytesIO()
+        self.serialize(out)
+        return out.getvalue()
+
+    def serialize(self, out, skip_source=False, fallback_lang=None):
         """convert to a string. double check that unicode is handled"""
-        return encode_if_needed_utf8(self.getoutput(skip_source, fallback_lang))
+        out.write(self.getoutput(skip_source, fallback_lang).encode(self.encoding))
 
     def getoutput(self, skip_source=False, fallback_lang=None):
         """converts all the lines back to tab-delimited form"""
         lines = []
         for oe in self.units:
             if len(oe.lines) > 2:
-                warnings.warn("contains %d lines (should be 2 at most): languages %r" % (len(oe.lines), oe.languages))
+                warnings.warn(
+                    "contains %d lines (should be 2 at most): languages %r"
+                    % (len(oe.lines), oe.languages)
+                )
                 oekeys = [line.getkey() for line in oe.lines]
-                warnings.warn("contains %d lines (should be 2 at most): keys %r" % (len(oe.lines), oekeys))
+                warnings.warn(
+                    "contains %d lines (should be 2 at most): keys %r"
+                    % (len(oe.lines), oekeys)
+                )
             oeline = oe.getoutput(skip_source, fallback_lang) + "\r\n"
             lines.append(oeline)
         return "".join(lines)
 
 
 class oomultifile:
-    """this takes a huge GSI file and represents it as multiple smaller files..."""
+    """this takes a huge GSI file and represents it as multiple smaller
+    files...
+    """
+
+    encoding = "utf-8"
 
     def __init__(self, filename, mode=None, multifilestyle="single"):
         """initialises oomultifile from a seekable inputfile or writable outputfile"""
         self.filename = filename
         if mode is None:
             if os.path.exists(filename):
-                mode = 'r'
+                mode = "r"
             else:
-                mode = 'w'
+                mode = "w"
         self.mode = mode
         self.multifilestyle = multifilestyle
         self.multifilename = os.path.splitext(filename)[0]
         self.multifile = open(filename, mode)
         self.subfilelines = {}
-        if mode == "r":
+        if mode.startswith("r"):
             self.createsubfileindex()
 
     def createsubfileindex(self):
@@ -348,7 +422,7 @@ class oomultifile:
         linenum = 0
         for line in self.multifile:
             subfile = self.getsubfilename(line)
-            if not subfile in self.subfilelines:
+            if subfile not in self.subfilelines:
                 self.subfilelines[subfile] = []
             self.subfilelines[subfile].append(linenum)
             linenum += 1
@@ -375,8 +449,7 @@ class oomultifile:
 
     def __iter__(self):
         """iterates through the subfile names"""
-        for subfile in self.listsubfiles():
-            yield subfile
+        yield from self.listsubfiles()
 
     def __contains__(self, pathname):
         """checks if this pathname is a valid subfile"""
@@ -397,7 +470,7 @@ class oomultifile:
     def openinputfile(self, subfile):
         """returns a pseudo-file object for the given subfile"""
         subfilesrc = self.getsubfilesrc(subfile)
-        inputfile = wStringIO.StringIO(subfilesrc)
+        inputfile = BytesIO(subfilesrc.encode())
         inputfile.filename = subfile
         return inputfile
 
@@ -405,8 +478,11 @@ class oomultifile:
         """returns a pseudo-file object for the given subfile"""
 
         def onclose(contents):
+            if isinstance(contents, bytes):
+                contents = contents.decode(self.encoding)
             self.multifile.write(contents)
             self.multifile.flush()
+
         outputfile = wStringIO.CatchStringOutput(onclose)
         outputfile.filename = subfile
         return outputfile
