@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2008-2011 Zuza Software Foundation
 #
@@ -24,43 +22,25 @@ This will eventually replace the older ts.py which only supports the older
 format. While converters haven't been updated to use this module, we retain
 both.
 
-`TS file format 4.3 <http://doc.qt.digia.com/4.3/linguist-ts-file-format.html>`_,
-`4.8 <http://qt-project.org/doc/qt-4.8/linguist-ts-file-format.html>`_,
-`5.0 <http://qt-project.org/doc/qt-5.0/qtlinguist/linguist-ts-file-format.html>`_.
+`TS file format 4.3 <http://doc.qt.io/archives/4.3/linguist-ts-file-format.html>`_,
+`4.8 <http://doc.qt.io/qt-4.8/linguist-ts-file-format.html>`_,
+`5 <http://doc.qt.io/qt-5/linguist-ts-file-format.html>`_.
 `Example <http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>`_.
 
-`Specification of the valid variable entries <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg>`_,
-`2 <http://qt-project.org/doc/qt-5.0/qtcore/qstring.html#arg-2>`_
+`Specification of the valid variable entries <http://doc.qt.io/qt-5/qstring.html#arg>`_,
+`2 <http://doc.qt.io/qt-5/qstring.html#arg-2>`_
 """
 
 from lxml import etree
 
 from translate.lang import data
 from translate.misc.multistring import multistring
-from translate.storage import base, lisa
+from translate.storage import lisa
 from translate.storage.placeables import general
 from translate.storage.workflow import StateEnum as state
 
-# TODO: handle translation types
 
-NPLURALS = {
-'jp': 1,
-'en': 2,
-'fr': 2,
-'lv': 3,
-'ga': 3,
-'cs': 3,
-'sk': 3,
-'mk': 3,
-'lt': 3,
-'ru': 3,
-'pl': 3,
-'ro': 3,
-'sl': 4,
-'mt': 4,
-'cy': 5,
-'ar': 6,
-}
+# TODO: handle translation types
 
 
 class tsunit(lisa.LISAunit):
@@ -69,7 +49,7 @@ class tsunit(lisa.LISAunit):
     rootNode = "message"
     languageNode = "source"
     textNode = ""
-    namespace = ''
+    namespace = ""
     rich_parsers = general.parsers
 
     S_OBSOLETE = state.OBSOLETE
@@ -78,10 +58,10 @@ class tsunit(lisa.LISAunit):
     S_TRANSLATED = state.UNREVIEWED
 
     statemap = {
-                "obsolete": S_OBSOLETE,
-                "unfinished": S_FUZZY,
-                "": S_TRANSLATED,
-                None: S_TRANSLATED,
+        "obsolete": S_OBSOLETE,
+        "unfinished": S_FUZZY,
+        "": S_TRANSLATED,
+        None: S_TRANSLATED,
     }
     """This maps the unit "type" attribute to state."""
 
@@ -92,7 +72,8 @@ class tsunit(lisa.LISAunit):
         S_TRANSLATED: (state.UNREVIEWED, state.MAX),
     }
 
-    statemap_r = dict((i[1], i[0]) for i in statemap.iteritems())
+    statemap_r = {i[1]: i[0] for i in statemap.items()}
+    _context = None
 
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
@@ -101,8 +82,8 @@ class tsunit(lisa.LISAunit):
         if purpose == "target":
             purpose = "translation"
         langset = etree.Element(self.namespaced(purpose))
-        #TODO: check language
-#        lisa.setXMLlang(langset, lang)
+        # TODO: check language
+        # lisa.setXMLlang(langset, lang)
 
         langset.text = text
         return langset
@@ -111,26 +92,36 @@ class tsunit(lisa.LISAunit):
         return self.xmlelement.find(self.namespaced(self.languageNode))
 
     def _gettargetnode(self):
-        return self.xmlelement.find(self.namespaced("translation"))
+        result = self.xmlelement.find(self.namespaced("translation"))
+        if result is not None:
+            return result
+        return etree.SubElement(self.xmlelement, self.namespaced("translation"))
 
     def getlanguageNodes(self):
         """We override this to get source and target nodes."""
+        return [
+            n for n in [self._getsourcenode(), self._gettargetnode()] if n is not None
+        ]
 
-        def not_none(node):
-            return not node is None
-        return filter(not_none, [self._getsourcenode(), self._gettargetnode()])
-
-    def getsource(self):
+    @lisa.LISAunit.source.getter
+    def source(self):
         # TODO: support <byte>. See bug 528.
-        sourcenode = self._getsourcenode()
+        text = self._getsourcenode().text
         if self.hasplural():
-            return multistring([sourcenode.text])
-        else:
-            return data.forceunicode(sourcenode.text)
-    source = property(getsource, lisa.LISAunit.setsource)
-    rich_source = property(base.TranslationUnit._get_rich_source, base.TranslationUnit._set_rich_source)
+            return multistring([text])
+        return text
 
-    def settarget(self, text):
+    @property
+    def target(self):
+        targetnode = self._gettargetnode()
+        if self.hasplural():
+            numerus_nodes = targetnode.findall(self.namespaced("numerusform"))
+            return multistring([node.text or "" for node in numerus_nodes])
+        else:
+            return targetnode.text or ""
+
+    @target.setter
+    def target(self, target):
         # This is a fairly destructive implementation. Don't assume that this
         # is necessarily correct in all regards, but it does deal with a lot of
         # cases. It is hard to deal with plurals.
@@ -138,15 +129,15 @@ class tsunit(lisa.LISAunit):
         # Firstly deal with reinitialising to None or setting to identical
         # string.
         self._rich_target = None
-        if self.gettarget() == text:
+        if self.target == target:
             return
         strings = []
-        if isinstance(text, multistring):
-            strings = text.strings
-        elif isinstance(text, list):
-            strings = text
+        if isinstance(target, multistring):
+            strings = target.strings
+        elif isinstance(target, list):
+            strings = target
         else:
-            strings = [text]
+            strings = [target]
         targetnode = self._gettargetnode()
         type = targetnode.get("type")
         targetnode.clear()
@@ -156,46 +147,32 @@ class tsunit(lisa.LISAunit):
             self.xmlelement.set("numerus", "yes")
             for string in strings:
                 numerus = etree.SubElement(targetnode, self.namespaced("numerusform"))
-                numerus.text = data.forceunicode(string) or u""
-                # manual, nasty pretty printing. See bug 1420.
-                numerus.tail = u"\n        "
+                numerus.text = string or ""
         else:
-            targetnode.text = data.forceunicode(text) or u""
-            targetnode.tail = u"\n    "
-
-    def gettarget(self):
-        targetnode = self._gettargetnode()
-        if targetnode is None:
-            etree.SubElement(self.xmlelement, self.namespaced("translation"))
-            return None
-        if self.hasplural():
-            numerus_nodes = targetnode.findall(self.namespaced("numerusform"))
-            return multistring([node.text or u"" for node in numerus_nodes])
-        else:
-            return data.forceunicode(targetnode.text) or u""
-    target = property(gettarget, settarget)
-    rich_target = property(base.TranslationUnit._get_rich_target, base.TranslationUnit._set_rich_target)
+            targetnode.text = target or ""
 
     def hasplural(self):
         return self.xmlelement.get("numerus") == "yes"
 
     def addnote(self, text, origin=None, position="append"):
         """Add a note specifically in the appropriate *comment* tag"""
-        if isinstance(text, str):
-            text = text.decode("utf-8")
         current_notes = self.getnotes(origin)
         self.removenotes(origin)
         if origin in ["programmer", "developer", "source code"]:
             note = etree.SubElement(self.xmlelement, self.namespaced("extracomment"))
         else:
-            note = etree.SubElement(self.xmlelement, self.namespaced("translatorcomment"))
+            note = etree.SubElement(
+                self.xmlelement, self.namespaced("translatorcomment")
+            )
         if position == "append":
-            note.text = "\n".join(filter(None, [current_notes, text.strip()]))
+            note.text = "\n".join(
+                item for item in [current_notes, text.strip()] if item
+            )
         else:
             note.text = text.strip()
 
     def getnotes(self, origin=None):
-        #TODO: consider only responding when origin has certain values
+        # TODO: consider only responding when origin has certain values
         comments = []
         if origin in ["programmer", "developer", "source code", None]:
             notenode = self.xmlelement.find(self.namespaced("extracomment"))
@@ -205,25 +182,22 @@ class tsunit(lisa.LISAunit):
             notenode = self.xmlelement.find(self.namespaced("translatorcomment"))
             if notenode is not None and notenode.text is not None:
                 comments.append(notenode.text)
-        return '\n'.join(comments)
+        return "\n".join(comments)
 
     def removenotes(self, origin=None):
         """Remove all the translator notes."""
         if origin in ["programmer", "developer", "source code", None]:
             note = self.xmlelement.find(self.namespaced("extracomment"))
-            if not note is None:
+            if note is not None:
                 self.xmlelement.remove(note)
         if origin in ["translator", None]:
             note = self.xmlelement.find(self.namespaced("translatorcomment"))
-            if not note is None:
+            if note is not None:
                 self.xmlelement.remove(note)
 
     def _gettype(self):
         """Returns the type of this translation."""
-        targetnode = self._gettargetnode()
-        if targetnode is not None:
-            return targetnode.get("type")
-        return None
+        return self._gettargetnode().get("type")
 
     def _settype(self, value=None):
         """Set the type of this translation."""
@@ -248,13 +222,17 @@ class tsunit(lisa.LISAunit):
             self._settype(None)
 
     def getid(self):
-        if self.source is None:
-            return None
         context_name = self.getcontext()
-        #XXX: context_name is not supposed to be able to be None (the <name>
+        if self.source is None and context_name is None:
+            return None
+
+        # XXX: context_name is not supposed to be able to be None (the <name>
         # tag is compulsary in the <context> tag)
         if context_name is not None:
-            return context_name + self.source
+            if self.source:
+                return context_name + self.source
+            else:
+                return context_name
         else:
             return self.source
 
@@ -269,26 +247,40 @@ class tsunit(lisa.LISAunit):
     def getcontextname(self):
         parent = self.xmlelement.getparent()
         if parent is None:
+            if self._context:
+                return self._context
             return None
         context = parent.find("name")
         if context is None:
             return None
         return context.text
 
+    def setcontext(self, value):
+        if value == self.getcontextname():
+            return
+        parent = self.xmlelement.getparent()
+        if parent is None:
+            self._context = value
+        # TODO: if unit is inserted to ts store
+        # it should put unit in correct context
+        # if unit is already in store it should move
+        # and adjust store as necessary
+
     def getcontext(self):
         contexts = [self.getcontextname()]
         commentnode = self.xmlelement.find(self.namespaced("comment"))
         if commentnode is not None and commentnode.text is not None:
             contexts.append(commentnode.text)
+        message_id = self.xmlelement.get("id")
+        if message_id is not None:
+            contexts.append(message_id)
         contexts = filter(None, contexts)
-        return '\n'.join(contexts)
+        return "\n".join(contexts)
 
     def addlocation(self, location):
-        if isinstance(location, str):
-            location = location.decode("utf-8")
         newlocation = etree.SubElement(self.xmlelement, self.namespaced("location"))
         try:
-            filename, line = location.split(':', 1)
+            filename, line = location.split(":", 1)
         except ValueError:
             filename = location
             line = None
@@ -304,22 +296,23 @@ class tsunit(lisa.LISAunit):
             line = location_tag.get("line")
             if line:
                 if location:
-                    location += ':' + line
+                    location += ":" + line
                 else:
                     location = line
-            locations.append(location)
+            if location:
+                locations.append(location)
         return locations
 
     def merge(self, otherunit, overwrite=False, comments=True, authoritative=False):
-        super(tsunit, self).merge(otherunit, overwrite, comments)
-        #TODO: check if this is necessary:
+        super().merge(otherunit, overwrite, comments)
+        # TODO: check if this is necessary:
         if otherunit.isfuzzy():
             self.markfuzzy()
         else:
             self.markfuzzy(False)
 
     def isobsolete(self):
-        return self._gettype() == "obsolete"
+        return self._gettype() == "obsolete" or self._gettype() == "vanished"
 
     def get_state_n(self):
         type = self._gettype()
@@ -330,6 +323,8 @@ class tsunit(lisa.LISAunit):
                 return self.S_FUZZY
             else:
                 return self.S_UNTRANSLATED
+        elif type == "vanished":
+            return self.S_OBSOLETE
         return self.statemap[type]
 
     def set_state_n(self, value):
@@ -345,22 +340,26 @@ class tsunit(lisa.LISAunit):
 
 class tsfile(lisa.LISAfile):
     """Class representing a TS file store."""
+
     UnitClass = tsunit
-    Name = _("Qt Linguist Translation File")
+    Name = "Qt Linguist Translation File"
     Mimetypes = ["application/x-linguist"]
     Extensions = ["ts"]
     rootNode = "TS"
     # We will switch out .body to fit with the context we are working on
     bodyNode = "context"
-    XMLskeleton = '''<!DOCTYPE TS>
+    XMLskeleton = """<!DOCTYPE TS>
 <TS>
 </TS>
-'''
-    namespace = ''
+"""
+    XMLindent = {"indent": "    ", "skip": ["TS"], "toplevel": False}
+    # For conformance with Qt output, write XML declaration with double quotes
+    XMLuppercaseEncoding = False
+    namespace = ""
 
     def __init__(self, *args, **kwargs):
         self._contextname = None
-        lisa.LISAfile.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def initbody(self):
         """Initialises self.body."""
@@ -384,9 +383,9 @@ class tsfile(lisa.LISAfile):
         :return: ISO code e.g. af, fr, pt_BR
         :rtype: String
         """
-        lang = data.normalize_code(self.header.get('sourcelanguage', "en"))
-        if lang == 'en-us':
-            return 'en'
+        lang = data.normalize_code(self.header.get("sourcelanguage", "en"))
+        if lang == "en-us":
+            return "en"
         return lang
 
     def gettargetlanguage(self):
@@ -395,7 +394,7 @@ class tsfile(lisa.LISAfile):
         :return: ISO code e.g. af, fr, pt_BR
         :rtype: String
         """
-        return data.normalize_code(self.header.get('language'))
+        return data.normalize_code(self.header.get("language"))
 
     def settargetlanguage(self, targetlanguage):
         """Set the target language for this .ts file to *targetlanguage*.
@@ -404,15 +403,17 @@ class tsfile(lisa.LISAfile):
         :type targetlanguage: String
         """
         if targetlanguage:
-            self.header.set('language', targetlanguage)
+            self.header.set("language", targetlanguage)
 
     def _createcontext(self, contextname, comment=None):
         """Creates a context node with an optional comment"""
-        context = etree.SubElement(self.document.getroot(), self.namespaced(self.bodyNode))
+        context = etree.SubElement(
+            self.document.getroot(), self.namespaced(self.bodyNode)
+        )
         name = etree.SubElement(context, self.namespaced("name"))
         name.text = contextname
         if comment:
-            comment_node = context.SubElement(context, "comment")
+            comment_node = etree.SubElement(context, "comment")
             comment_node.text = comment
         return context
 
@@ -423,8 +424,7 @@ class tsfile(lisa.LISAfile):
     def _getcontextnames(self):
         """Returns all contextnames in this TS file."""
         contextnodes = self.document.findall(self.namespaced("context"))
-        contextnames = [self.getcontextname(contextnode) for contextnode in contextnodes]
-        return contextnames
+        return [self.getcontextname(contextnode) for contextnode in contextnodes]
 
     def _getcontextnode(self, contextname):
         """Returns the context node with the given name."""
@@ -434,30 +434,34 @@ class tsfile(lisa.LISAfile):
                 return contextnode
         return None
 
-    def addunit(self, unit, new=True, contextname=None, createifmissing=True):
+    def addunit(
+        self, unit, new=True, contextname=None, comment=None, createifmissing=True
+    ):
         """Adds the given unit to the last used body node (current context).
 
-        If the contextname is specified, switch to that context (creating it
-        if allowed by createifmissing)."""
+        If the contextname is specified, switch to that context (creating it if
+        allowed by createifmissing).
+        """
         if contextname is None:
             contextname = unit.getcontextname()
 
         if self._contextname != contextname:
-            if not self._switchcontext(contextname, createifmissing):
+            if not self._switchcontext(contextname, comment, createifmissing):
                 return None
-        super(tsfile, self).addunit(unit, new)
-#        lisa.setXMLspace(unit.xmlelement, "preserve")
+        super().addunit(unit, new)
+        #        lisa.setXMLspace(unit.xmlelement, "preserve")
         return unit
 
-    def _switchcontext(self, contextname, createifmissing=False):
+    def _switchcontext(self, contextname, comment, createifmissing=False):
         """Switch the current context to the one named contextname, optionally
-        creating it if it doesn't exist."""
+        creating it if it doesn't exist.
+        """
         self._contextname = contextname
         contextnode = self._getcontextnode(contextname)
         if contextnode is None:
             if not createifmissing:
                 return False
-            contextnode = self._createcontext(contextname)
+            contextnode = self._createcontext(contextname, comment)
 
         self.body = contextnode
         if self.body is None:
@@ -465,25 +469,34 @@ class tsfile(lisa.LISAfile):
         return True
 
     def nplural(self):
-        lang = self.header.get("language")
-        if lang in NPLURALS:
-            return NPLURALS[lang]
-        else:
+        lang = data.get_language(self.header.get("language"))
+        if lang is None:
             return 1
+        return lang[1]
 
-    def __str__(self):
-        """Converts to a string containing the file's XML.
+    def serialize_hook(self, treestring):
+        pos = 0
+        out = []
+        while pos >= 0:
+            nextpos = treestring.find(b"<", pos)
+            out.append(
+                treestring[pos:nextpos]
+                .replace(b"'", b"&apos;")
+                .replace(b'"', b"&quot;")
+                .replace(b"\xc2\xa0", b"&#xa0;")
+            )
+            pos = nextpos
+            nextpos = treestring.find(b">", pos)
+            out.append(treestring[pos:nextpos])
+            pos = nextpos
+        out.append(treestring[pos:])
+        return b"".join(out)
 
-        We have to override this to ensure mimic the Qt convention:
-            - no XML decleration
-            - plain DOCTYPE that lxml seems to ignore
-        """
-        # A bug in lxml means we have to output the doctype ourselves. For
-        # more information, see:
-        # http://codespeak.net/pipermail/lxml-dev/2008-October/004112.html
-        # The problem was fixed in lxml 2.1.3
-        output = etree.tostring(self.document, pretty_print=True,
-                                xml_declaration=False, encoding='utf-8')
-        if not "<!DOCTYPE TS>" in output[:30]:
-            output = "<!DOCTYPE TS>" + output
-        return output
+    def serialize(self, out):
+        """Write the XML document to a file."""
+        root = self.document.getroot()
+        for e in root.xpath(
+            "//*[not(./node()) and not(text()) and not(name() = 'location')]"
+        ):
+            e.text = ""
+        super().serialize(out)
